--[[
********************************************************
Script for USB RC-Remote Aurico RC-CHDK2
Special version using only one RC-channel for shoot&zoom
Requires CHDK 1.3 v3442 or later for USB high speed mode
Version: 1.00 Beta 1
Date: 2016/08/21
Licence: GPL
Author: Werner_O
THX to msl
********************************************************
@title RC-CHDK1 v1.00b1
@chdk_version 1.3

@param a Enable AutoMode USB-Port
 @default a 1
 @range a 0 1
@param t Zoom In/Out Time (100ms)
 @default t 8
 @range t 3 20
@param l USB Time Limit (10ms)
 @default l 13
 @range l 10 40
@param x Start in USB Test-Mode
 @default x 0
 @range x 0 1
@param r Result Report Time (s)
 @default r 3
 @range r 1 10
]]

--Console layout:
set_console_layout(0, 0, 26, 8)

--Predefinition of the used variables:
AutoMode = a
TimeLimit = l
InOutTime = t * 100
TestMode = x
TestResultTime = r * 1000
zoom_step = get_zoom()
MaxZoom = get_zoom_steps() - 1     --amount of available zoom steps
current_tick2 = get_tick_count()
last_tick2 = get_tick_count()

--Subfunction restore() used at script interrupt:
function restore()
    cls()
    print("Aborting Script:")
    if AutoMode == 1 then
        set_config_value(121,0)   --Disable USB remote
        print("USB remote disabled")
    end
    if high_speed then            --Disable USB high speed mode if automatically enabled
        if type(set_remote_timing) == "function" then set_remote_timing(0)  
        elseif type(enable_remote_hp_timer) == "function" then enable_remote_hp_timer(0)
        elseif type(enable_highspeed_usb) == "function" then enable_highspeed_usb(0)
        end
        print("High speed mode disabled")
    end
    print("Finished!")
    sleep(3000)
    console_redraw()
end

--Subfunction wait_message():
function wait_message()
    print("Waiting for USB Impulse...")
    print("(Press MENU to EXIT)")
    last_tick = get_tick_count()
end

--Subfunction zoom_message():
function zoom_message()
    zoom_step = get_zoom()
    print(string.format("#Zoom-Step: %d of %d", zoom_step, MaxZoom))
end

--Subfunction command_shoot():
function command_shoot()
    cls()
    count = get_exp_count()
    shoot()
    repeat sleep(10) until get_exp_count() ~= count  --wait until camera has stored the picture
    print("Shooting successful")
    wait_message()
end

--Subfunction command_zoomin():
function command_zoomin()
    last_zoom = get_zoom()
    cls()
    if get_zoom() < MaxZoom then      --is current zoom position lower than max.?
        print("Zoom-In +1 Step...")
        set_zoom(last_zoom + 1)
        sleep(500)
        zoom_message()
    else
        zoom_message()
        print("MAX. ZOOM POSITION!")
    end
    wait_message()
end

--Subfunction command_zoomout():
function command_zoomout()
    last_zoom = get_zoom()
    cls()
    if get_zoom() > 0 then            --is current zoom position higher than min.?
        print("Zoom-Out -1 Step...")
        set_zoom(last_zoom - 1)
        sleep(500)
        zoom_message()
    else
        zoom_message()
        print("MIN. ZOOM POSITION!")
    end
    wait_message()
end

--Start main script:

--Enable Shooting Mode when started in Playback Mode
if get_mode() == false then       --True at found playback mode
    print("Activate Record Mode...")
    set_record(1)
    sleep(1000)
    while get_mode() == false do sleep(10) end
    print("Record Mode activated!")
    sleep(3000)
    cls()
end

print("Script started - State:")
if AutoMode == 1 then
    set_config_value(121,1)        --enable USB remote
    print("USB remote: Enabled")
end
high_speed = false    --Enable USB high speed mode if possible, available since CHDK-DE 1.3 v3442
--Command for CHDK 1.3 >= v3791 or CHDK 1.4/1.5 (boolean):
if type(set_remote_timing) == "function" then high_speed = set_remote_timing(1000)
--Command for CHDK 1.3 >= v3585 to < v3791 (boolean):
elseif type(enable_remote_hp_timer) == "function" then high_speed = enable_remote_hp_timer(1000)
--Command for CHDK 1.3 >= v3442 to < v3585 (number):
elseif type(enable_highspeed_usb) == "function" then enable_highspeed_usb(1000) high_speed = true
end
if high_speed then
    TimeLimit = l * 10
    print("High speed mode: Enabled")
else
    print("High speed mode: n/a")
end

if TestMode == 1 then
    print("USB Test Mode: Enabled")
    sleep(3000)
    console_redraw()
    sleep(3000)
    cls()
    print("USB Test Mode:")
    wait_message()
else
    sleep(3000)
    console_redraw()
    sleep(3000)
    cls()
    wait_message()
end

repeat   --Start main loop
    current_tick = get_tick_count()
    if ((current_tick - last_tick) >= 2900) then   --redraw console after 2.9 sec
        last_tick = current_tick
        console_redraw()
    end
    u = get_usb_power()
    if u > 0 then
        if TestMode == 1 then         --true in USB test mode
            cls()
            print("USB Impulse recognized:")
            if high_speed then
                print(string.format("Impulse Length = %d ms", u))
            else
                print(string.format("Impulse Length = %d0 ms", u))
            end
            print(string.format("(Script paused for %d sec)", p))
            sleep(TestResultTime)         --pause to display result
            print("**************************")
            print("USB Test Mode:")
            wait_message()
        else                          --true in shooting mode
            if u < TimeLimit then     --true if USB impulse is shorter than TimeLimit
                command_shoot()
            else                      --true if USB impulse is longer than TimeLimit
                last_tick2 = get_tick_count()
                print("(Wait for ZoomOut Impulse)")
                repeat
                    current_tick2 = get_tick_count()
                    v = get_usb_power()
                until (v >= TimeLimit) or ((current_tick2 - last_tick2) >= InOutTime)
                if (v >= TimeLimit) then
                    command_zoomout()
                else
                    command_zoomin()
                end
            end
        end
    end
until is_pressed("menu")

restore()
